import Flutter
import UIKit
import PinchSDK
import PureSDK

public class SwiftPinchPlugin: NSObject, FlutterPlugin {
  public static func register(with registrar: FlutterPluginRegistrar) {
    let channel = FlutterMethodChannel(name: "com.fluxloop.pinch/sdk", binaryMessenger: registrar.messenger())
    let instance = SwiftPinchPlugin()
    registrar.addMethodCallDelegate(instance, channel: channel)
  }

  public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
    switch call.method {
    case "start":
        Pinch.start()
        break
    case "stop":
        Pinch.stop()
        break
    case "setMessagingId":
        result(true)
        break
    case "getStarted":
        result(Pinch.started)
        break
    case "addCustomEvent":
        guard let args = call.arguments as? [String: Any] else {
            result(FlutterError.init(code: "INVALID_ARGS", message: "Invalid arguments supplied", details: nil))
            return
        }
        let json = args["json"] as? String
        let type = args["type"] as? String
        if (json == nil || type == nil) {
            result(false)
            return
        }
        if let dict = (convertToDictionary(json: json!)) {
            Pinch.sendCustomData(type: type, json: dict) { (success) in
                result(success)
                return
            }
        } else {
            result(false)
            return
        }
        break
    case "setMetadata":
        result(true)
        break
    case "sendDemographicProfile":
        guard let args = call.arguments as? [String: Any] else {
            result(FlutterError.init(code: "INVALID_ARGS", message: "Invalid arguments supplied", details: nil))
            return
        }
        let birthYear = args["birthYear"] as? Int
        let gender = args["gender"] as? String

        if (birthYear == nil || gender == nil) {
            result(false)
            return
        }

        var dict = Dictionary<String, Any>()
        dict["birthYear"] = birthYear
        dict["gender"] = gender

        Pinch.sendCustomData(type: type, json: dict) { (success) in
                result(success)
                return
            }
        break
    default:
        result(FlutterError.init(code: "INVALID_METHOD", message: "Method does not exist", details: nil))
    }
    result(true)
  }

    private func convertToDictionary(json: String) -> [String: Any]? {
        if let data = json.data(using: .utf8) {
            do {
                return try JSONSerialization.jsonObject(with: data, options: []) as? [String: Any]
            } catch {
              //
            }
        }
        return nil
    }

    private func genderToInt(gender: String) -> Int {
        switch gender {
        case "male":
            return 1
        case "female":
            return 2
        default:
            return 0
        }
    }
}
